﻿using System;

using Akka.Actor;

using TestLibrary;

namespace TestClient
{
    /// <summary>
    /// 채팅 클라이언트 액터
    /// </summary>
    public class ChatClientActor : ReceiveActor, ILogReceive
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Field
        ////////////////////////////////////////////////////////////////////////////////////////// Private

        #region Field

        /// <summary>
        /// 액터 선택
        /// </summary>
        private readonly ActorSelection actorSelection = Context.ActorSelection("akka.tcp://TestServer@localhost:9999/user/ChatServer");

        /// <summary>
        /// 닉네임
        /// </summary>
        private string nickName = "Roggan";

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Constructor
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 생성자 - ChatClientActor()

        /// <summary>
        /// 생성자
        /// </summary>
        public ChatClientActor()
        {
            Receive<ConnectRequest>
            (
                connectRequest =>
                {
                    Console.WriteLine("Connecting....");

                    this.actorSelection.Tell(connectRequest);
                }
            );

            Receive<ConnectResponse>
            (
                connectResponse =>
                {
                    Console.WriteLine("Connected!");

                    Console.WriteLine(connectResponse.Message);
                }
            );

            Receive<NickNameRequest>
            (
                nickNameRequest =>
                {
                    nickNameRequest.UserName = this.nickName;

                    Console.WriteLine($"Changing nick to {nickNameRequest.NewUserName}");

                    this.nickName = nickNameRequest.NewUserName;

                    this.actorSelection.Tell(nickNameRequest);
                }
            );

            Receive<NickNameResponse>
            (
                nickNameRequest =>
                {
                    Console.WriteLine($"{nickNameRequest.UserName} is now known as {nickNameRequest.NewUserName}");
                }
            );

            Receive<SayRequest>
            (
                sayRequest =>
                {
                    sayRequest.UserName = this.nickName;

                    this.actorSelection.Tell(sayRequest);
                }
            );

            Receive<SayResponse>
            (
                sayResponse =>
                {
                    Console.WriteLine($"{sayResponse.UserName} : {sayResponse.Message}");
                }
            );
        }

        #endregion
    }
}