﻿using System;
using System.Linq;

using Akka.Actor;
using Akka.Configuration;

using TestLibrary;

namespace TestClient
{
    /// <summary>
    /// 프로그램
    /// </summary>
    class Program
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Static
        //////////////////////////////////////////////////////////////////////////////// Private

        #region 프로그램 시작하기 - Main()

        /// <summary>
        /// 프로그램 시작하기
        /// </summary>
        private static void Main()
        {
            string hocon = @"
akka
{  
    actor
    {
        provider = remote
    }
    remote
    {
        dot-netty.tcp
        {
            port     = 0
            hostname = localhost
        }
    }
}
";

            Config config = ConfigurationFactory.ParseString(hocon);

            using(ActorSystem actorSystem = ActorSystem.Create("TestClient", config))
            {
                IActorRef actorRef = actorSystem.ActorOf(Props.Create<ChatClientActor>());

                actorRef.Tell(new ConnectRequest() { UserName = "Roggan", });

                while(true)
                {
                    string input = Console.ReadLine();

                    if(input.StartsWith("/"))
                    {
                        string[] partArray = input.Split(' ');
                        string   command   = partArray[0].ToLowerInvariant();
                        string   rest      = string.Join(" ", partArray.Skip(1));

                        if(command == "/nick")
                        {
                            actorRef.Tell(new NickNameRequest { NewUserName = rest });
                        }

                        if(command == "/exit")
                        {
                            Console.WriteLine("exiting");

                            break;
                        }
                    }
                    else
                    {
                        actorRef.Tell(new SayRequest() { Message = input });
                    }
                }

                actorSystem.Terminate().Wait();
            }
        }

        #endregion
    }
}