﻿using System.Collections.Generic;

using Akka.Actor;

using TestLibrary;

namespace TestServer
{
    /// <summary>
    /// 채팅 서버 액터
    /// </summary>
    public class ChatServerActor : ReceiveActor, ILogReceive
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Field
        ////////////////////////////////////////////////////////////////////////////////////////// Private

        #region Field

        /// <summary>
        /// 액터 참조 해시 세트
        /// </summary>
        private readonly HashSet<IActorRef> actorRefHashSet = new HashSet<IActorRef>();

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Constructor
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 생성자 - ChatServerActor()

        /// <summary>
        /// 생성자
        /// </summary>
        public ChatServerActor()
        {
            Receive<ConnectRequest>
            (
                connectRequest =>
                {
                    this.actorRefHashSet.Add(Sender);

                    Sender.Tell
                    (
                        new ConnectResponse
                        {
                            Message = "Hello and welcome to Akka.NET chat example",
                        },
                        Self
                    );
                }
            );

            Receive<NickNameRequest>
            (
                nickNameRequest =>
                {
                    NickNameResponse nickNameResponse = new NickNameResponse
                    {
                        UserName    = nickNameRequest.UserName,
                        NewUserName = nickNameRequest.NewUserName,
                    };

                    foreach(IActorRef actorRef in this.actorRefHashSet)
                    {
                        actorRef.Tell(nickNameResponse, Self);
                    }
                }
            );

            Receive<SayRequest>
            (
                sayRequest =>
                {
                    SayResponse sayResponse = new SayResponse
                    {
                        UserName = sayRequest.UserName,
                        Message  = sayRequest.Message,
                    };

                    foreach(IActorRef actorRef in this.actorRefHashSet)
                    {
                        actorRef.Tell(sayResponse, Self);
                    }
                }
            );
        }

        #endregion
    }
}